-------------------------------------------------------------------------------
-- Localized Lua globals
-------------------------------------------------------------------------------
local _G = getfenv(0)

local string = _G.string
local format = string.format
local strsplit = string.split

local math = _G.math
local floor = math.floor

local table = _G.table
local wipe = table.wipe
local tinsert = table.insert
local tremove = table.remove

local ipairs, pairs = _G.ipairs, _G.pairs

-------------------------------------------------------------------------------
-- Localized Blizzard API
-------------------------------------------------------------------------------
local UnitClass, UnitLevel, UnitName = _G.UnitClass, _G.UnitLevel, _G.UnitName
local GetItemInfo, GetItemGem = _G.GetItemInfo, _G.GetItemGem
local GetInventoryItemLink = _G.GetInventoryItemLink

-------------------------------------------------------------------------------
-- AddOn namespace
-------------------------------------------------------------------------------
local NAME = "GearGauge"
local GearGauge = LibStub("AceAddon-3.0"):NewAddon(NAME, "AceEvent-3.0", "AceConsole-3.0")

local LDB = LibStub("LibDataBroker-1.1")
local QTC = LibStub("LibQTipClick-1.1")
local LBC = LibStub("LibBabble-Class-3.0"):GetLookupTable()
local LIB = LibStub("LibItemBonus-2.0")

local dev = false
--[===[@debug@
dev = true
--@end-debug@]===]
local L = LibStub("AceLocale-3.0"):GetLocale(NAME, "enUS", true, dev)

local DataObj
local tooltip

local version = GetAddOnMetadata(NAME, "Version")
version = string.gsub(version, "@project.version@", " - Development")

-------------------------------------------------------------------------------
-- Constants
-------------------------------------------------------------------------------
local TBC_MAXLEV = 60
local WOTLK_MAXLEV = 71
local IS_ENCHANTER = false

local COLOR_RED = "|cffff0000"

local EQUIPMENT = {
	["Head"]		= { minlev = 30,	name = _G["INVTYPE_HEAD"],		mult = 0.8125 },
	["Neck"]		= { minlev = 30,	name = _G["INVTYPE_NECK"],		mult = 0.55 },
	["Shoulder"]		= { minlev = 20,	name = _G["INVTYPE_SHOULDER"],		mult = 0.75 },
	["Shirt"]		= { minlev = 1,		name = _G["INVTYPE_BODY"],		mult = 0 },
	["Chest"]		= { minlev = 1,		name = _G["INVTYPE_CHEST"],		mult = 1 },
	["Waist"]		= { minlev = 1,		name = _G["INVTYPE_WAIST"],		mult = 0.5625 },
	["Legs"]		= { minlev = 1,		name = _G["INVTYPE_LEGS"],		mult = 0.875 },
	["Feet"]		= { minlev = 1,		name = _G["INVTYPE_FEET"],		mult = 0.6875 },
	["Wrist"]		= { minlev = 1,		name = _G["INVTYPE_WRIST"],		mult = 0.4375 },
	["Hands"]		= { minlev = 1,		name = _G["INVTYPE_HAND"],		mult = 0.625 },
	["Finger0"]		= { minlev = 20,	name = _G["INVTYPE_FINGER"],		mult = 0.55 },
	["Finger1"]		= { minlev = 20,	name = _G["INVTYPE_FINGER"],		mult = 0.55 },
	["Trinket0"]		= { minlev = 50,	name = _G["INVTYPE_TRINKET"],		mult = 0.75 },
	["Trinket1"]		= { minlev = 50,	name = _G["INVTYPE_TRINKET"],		mult = 0.75 },
	["Back"]		= { minlev = 1,		name = _G["INVTYPE_CLOAK"],		mult = 0.48 },
	["MainHand"]		= { minlev = 1,		name = _G["INVTYPE_WEAPONMAINHAND"],	mult = 0.50 },
	["SecondaryHand"]	= { minlev = 1,		name = _G["INVTYPE_WEAPONOFFHAND"],	mult = 0.50 },
	["Ranged"]		= { minlev = 1,		name = _G["INVTYPE_RANGED"],		mult = 0.35 },
	["Tabard"]		= { minlev = 1,		name = _G["INVTYPE_TABARD"],		mult = 0 },
}

local ORDERED_EQUIPMENT = {
	[1]	= "Head",	[2]	= "Neck",		[3]	= "Shoulder",
	[4]	= "Shirt",	[5]	= "Chest",		[6]	= "Waist",
	[7]	= "Legs",	[8]	= "Feet",		[9]	= "Wrist",
	[10]	= "Hands",	[11]	= "Finger0",		[12]	= "Finger1",
	[13]	= "Trinket0",	[14]	= "Trinket1",		[15]	= "Back",
	[16]	= "MainHand",	[17]	= "SecondaryHand",	[18]	= "Ranged",
	[19]	= "Tabard"
}

local TWO_HANDED_WEAPON = {
	[L["Fishing Poles"]] = true,
	[L["Polearms"]] = true,
	[L["Staves"]] = true,
	[L["Two-Handed Axes"]] = true,
	[L["Two-Handed Maces"]] = true,
	[L["Two-Handed Swords"]] = true,
}

local GEM_TYPES = {
	"Blue",
	"Meta",
	"Red",
	"Yellow"
}

-------------------------------------------------------------------------------
-- Variables
-------------------------------------------------------------------------------
local char_equip = {}		-- Character's equipment. Generated when AddOn is loaded or state is changed.
local db

-------------------------------------------------------------------------------
-- Table cache
-------------------------------------------------------------------------------
local AcquireTable, ReleaseTable
do
	local table_cache = {}

	-- Returns a table
	function AcquireTable()
		local tbl = tremove(table_cache) or {}
		return tbl
	end

	-- Cleans the table and stores it in the cache
	function ReleaseTable(tbl)
		if not tbl then return end
		wipe(tbl)
		tinsert(table_cache, tbl)
	end
end	-- do

-------------------------------------------------------------------------------
-- Local functions
-------------------------------------------------------------------------------
local function AverageItemLevels(equip_list)
	local level_cnt = 0
	local item_cnt = 0

	for slot in pairs(EQUIPMENT) do
		local item = equip_list[slot]

		if item.level > 0 then
			level_cnt = level_cnt + item.level
			item_cnt = item_cnt + 1
		end
	end
	if item_cnt == 0 then return 0 end

	local avg = floor(level_cnt / item_cnt)
	return avg
end

-------------------------------------------------------------------------------
-- Item status generation
-------------------------------------------------------------------------------
local BuildGearData
do
	-------------------------------------------------------------------------------
	-- Check to see if an empty slot is OK given the unit's level/class.
	-------------------------------------------------------------------------------
	local function IsEmptySlotOK(slot, level, class)
		if slot == "SecondaryHand"  then
			if level < 5 and (class == LBC["Paladin"] or class == LBC["Shaman"] or class == LBC["Warrior"]) then	-- Shields
				return true
			else
				return false
			end
		end

		if slot == "Ranged" then
			if  level < 52 and (class == LBC["Druid"] or class == LBC["Paladin"] or class == LBC["Shaman"]) then	-- Idols, Librams, Totems
				return true
			elseif level < 5 and (class == LBC["Mage"] or class == LBC["Priest"] or class == LBC["Warlock"]) then	-- Wands
				return true
			else
				return false
			end
		end
		return EQUIPMENT[slot].minlev > level
	end

	local function GetEmptySockets(item)
		if not item then return end

		if not item.link or not item.info or not item.info.bonuses then
			item.emptyBlue, item.emptyMeta, item.emptyRed, item.emptyYellow = 0, 0, 0, 0
			return
		end
		local bonuses = item.info.bonuses

		item.emptyBlue = LIB:GetBonus("EMPTY_SOCKET_BLUE", bonuses) or 0
		item.emptyMeta = LIB:GetBonus("EMPTY_SOCKET_META", bonuses) or 0
		item.emptyRed = LIB:GetBonus("EMPTY_SOCKET_RED", bonuses) or 0
		item.emptyYellow = LIB:GetBonus("EMPTY_SOCKET_YELLOW", bonuses) or 0
	end

	-- Item subtypes which are not enchantable
	local NON_ENCHANTABLE = {
		[L["Idols"]] = true,
		[L["Librams"]] = true,
		[L["Miscellaneous"]] = true,
		[L["Other"]] = true,
		[L["Sigils"]] = true,
		[L["Thrown"]] = true,
		[L["Totems"]] = true,
		[L["Wands"]] = true,
	}
	local prev_stype	-- Placeholder in case previous slot contained a two-handed weapon.

	-------------------------------------------------------------------------------
	-- Assign values to an item for later use.
	-------------------------------------------------------------------------------
	local function ConstructItem(item_link, slot, unit)
		local unit_level = UnitLevel(unit)
		local item = AcquireTable()

		-- Default values here
		item.slot, item.rating, item.level = slot, 0, 0
		item.emptyMeta, item.emptyRed, item.emptyYellow, item.emptyBlue = 0, 0, 0, 0
		item.ench_id = -1

		if not item_link then
			local two_handed = slot == "SecondaryHand" and TWO_HANDED_WEAPON[prev_stype]
			local is_ok = IsEmptySlotOK(slot, unit_level, UnitClass(unit))

			if not two_handed and not is_ok then
				item.rating = - EQUIPMENT[slot].mult
			end
			item.empty_ok = is_ok
			prev_stype = nil
			return item
		end
		item.name, item.link, item.rarity, item.level, item.minlev, item.type, item.stype, item.stacksize, item.equiploc, item.texture = GetItemInfo(item_link)
		prev_stype = item.stype

		-- Set the level of Heirloom items depending on the player's level
		if item.rarity == 7 then
			if unit_level <= TBC_MAXLEV then
				item.level = unit_level
			elseif unit_level > TBC_MAXLEV and unit_level <= WOTLK_MAXLEV then
				item.level = 3 * (unit_level - 64) + 90
			elseif unit_level > WOTLK_MAXLEV then
				item.level = 4 * (unit_level - 73) + 142
			end
		end
		if item.level <= 1 or slot == "Tabard" then return item end

		item.info = LIB:ScanItemLink(item_link)

		local points = 0

                if item.info.bonuses then
                        for k, v in pairs(item.info.bonuses) do
				local bonus = LIB:GetBonus(k, item.info.bonuses, unit_level)

				if bonus < 1 then GearGauge:Print("No bonus for "..item.name) else
					points = points + bonus
				end
                        end
                end
		local mult = EQUIPMENT[slot].mult

		if TWO_HANDED_WEAPON[item.stype] then
			mult = mult + mult
		end
		points = ((points + (item.level / unit_level)) * 0.1) * mult

		local rating = points
		local _, _, ench_id = strsplit(":", item_link)		-- Grab the item's enchant id, if enchanted.

		if slot ~= "Waist" and slot ~= "Shirt" and slot ~= "Tabard" then
			item.ench_id = ench_id

			if ench_id == nil or ench_id == "0" then
				if ((slot == "Finger0" or slot == "Finger1") and IS_ENCHANTER) or not NON_ENCHANTABLE[item.stype] then
					rating = rating - (points * 0.25)
				else
					item.ench_id = -1
				end
			end
		end

		-- Get gem information
		GetEmptySockets(item)

		for k, v in ipairs(GEM_TYPES) do
			if item["empty"..v] > 0 then
				rating = rating - ((points * 0.125) * item["empty"..v])
			end
		end
		item.rating = rating

		return item
	end

	-------------------------------------------------------------------------------
	-- List of items equipped by unit.
	-------------------------------------------------------------------------------
	function BuildGearData(unit, item_list, link_list)
		for idx, slot in ipairs(ORDERED_EQUIPMENT) do
			ReleaseTable(item_list[slot])
		end
		wipe(item_list)

		local rating = 0

		prev_stype = nil	-- Nil this out - it's used in ConstructItem().

		for idx, slot in ipairs(ORDERED_EQUIPMENT) do
			local link = GetInventoryItemLink(unit, EQUIPMENT[slot].id)
			local item = ConstructItem(link, slot, unit)

			rating = rating + item.rating
			item_list[slot] = item

			if link_list then
				link_list[slot] = link
			end
		end
		return rating
	end
end -- do

do
	function GearGauge:Scan(unit)
		if unit == "player" then return end

		local enemies = UnitFactionGroup(unit) ~= UnitFactionGroup("player")
		local prefs = db.profile.scanning

		if enemies and not prefs.enemies then return end

		if InCombatLockdown() or not CheckInteractDistance("target", 1)
			or not (UnitExists(unit) and UnitIsPlayer(unit)
				and (UnitLevel(unit) >= prefs.minlevel) and (UnitLevel(unit) <= prefs.maxlevel)) then return end

		if not CanInspect(unit, false) then
			GearGauge:Print(COLOR_RED..UnitName(unit)..": "..ERR_PLAYER_WRONG_FACTION.."|r")
			return
		end
		NotifyInspect(unit)

		local unit_name = UnitName(unit)
		local item_list = AcquireTable()
		local target = db.realm.entries[unit_name]

		if not target then
			target = AcquireTable()

			target.class = UnitClass(unit)
			target.faction = UnitFactionGroup(unit)
			target.gender = UnitSex(unit)
			target.guild = GetGuildInfo(unit) or "("..NOT_APPLICABLE..")"
			target.link_list = AcquireTable()
			target.level = UnitLevel(unit)
			target.race = UnitRace(unit)
			target.scanned = time()

			db.realm.entries[unit_name] = target
		else
			local scanned = time() - target.scanned

			-- If they haven't timed out yet, don't rescan.
			if scanned < (60 * prefs.delay) then
				return
			else
				target.scanned = time()
			end
		end
		target.rating = BuildGearData(unit, item_list, target.link_list)

		local equip_level = AverageItemLevels(item_list)

		for idx, slot in ipairs(ORDERED_EQUIPMENT) do
			ReleaseTable(item_list[slot])
		end
		ReleaseTable(item_list)

		if not prefs.notify then return end

		local color_str = ""
		if enemies then color_str = COLOR_RED end

		self:Print(color_str..unit_name.." "..RATING..": "..format("%.2f", target.rating)..". "..ITEMS.." "..LEVEL..": "..equip_level)
	end
end -- do

-------------------------------------------------------------------------------
-- Event handlers
-------------------------------------------------------------------------------
-- Check to see if entries should be pruned, and update teh DataObj text.
local function UpdateData()
	local scanned

	for k, v in pairs(db.realm.entries) do
		for key, val in pairs(db.realm.entries[k]) do
			if key == "scanned" then
				scanned = time() - val

				if scanned >= (86400 * db.profile.scanning.persistence) then
					ReleaseTable(db.realm.entries[k])
					db.realm.entries[k] = nil
				end
			end
		end
	end
	DataObj.text = format("%.2f", BuildGearData("player", char_equip))
end

function GearGauge:PLAYER_LEVEL_UP()
	UpdateData()
end

function GearGauge:PLAYER_TARGET_CHANGED()
	self:Scan("target")
end

function GearGauge:UNIT_INVENTORY_CHANGED()
	UpdateData()
end

-------------------------------------------------------------------------------
-- Tooltip functions
-------------------------------------------------------------------------------
local GetItemStatus
do
	-- Returns a string based on whether or not the item is enchanted
	function GetItemStatus(item)
		if not item.link then return HELPFRAME_ITEM_TITLE end

		local ench_id = item.ench_id
		local retstr = ""

		if ench_id ~= -1 and (ench_id == nil or ench_id == "0") then
			retstr = retstr.."|TInterface\\Icons\\Trade_Engraving:22:22|t"
		end
		local socket_tex = "|TInterface\\ItemSocketingFrame\\UI-EmptySocket-"

		for k, v in ipairs(GEM_TYPES) do
			if item["empty"..v] > 0 then
				retstr = retstr..socket_tex..v..":22:22|tx"..item["empty"..v]
			end
		end
		return retstr
	end
end --do

local ItemOnEnter, ItemOnLeave
do
	local QTip = LibStub("LibQTip-1.0")
	local bonus_tip

	function ItemOnEnter(event, cell, arg)
		if not arg then return end
		QTC.OnEnter(event, cell, arg)

		bonus_tip = QTip:Acquire(NAME.."BonusTip", 2, "LEFT", "LEFT")
		bonus_tip:ClearAllPoints()
		bonus_tip:SetPoint("TOPRIGHT", tooltip, "TOPLEFT")
		bonus_tip:Clear()

		for k, v in pairs(arg) do
			bonus_tip:AddLine(k, v)
		end

		if TipTac and TipTac.AddModifiedTip then TipTac:AddModifiedTip(bonus_tip, true) end

		bonus_tip:Show()
	end

	function ItemOnLeave(event, cell, arg)
		if not bonus_tip then return end
		QTC.OnLeave(event, cell, arg)
		bonus_tip = QTip:Release(bonus_tip)
	end
end	-- do

local function BuildTooltip(anchor, unit)
	local prefs = db.profile.tooltip

	tooltip = QTC:Acquire(NAME.."Tooltip", 3, "LEFT", "LEFT", "LEFT")
	tooltip:SetCallback("OnEnter", ItemOnEnter)
	tooltip:SetCallback("OnLeave", ItemOnLeave)
	tooltip:SetAutoHideDelay(prefs.timer, anchor)
	tooltip:SmartAnchorTo(anchor)
	tooltip:Clear()

	if TipTac and TipTac.AddModifiedTip then TipTac:AddModifiedTip(tooltip, true) end

	unit = unit or "player"
	local equip_level = AverageItemLevels(unit == "player" and char_equip or db.realm.entries[UnitName(unit)])
	local rating_tot = 0
	local line, column

	line, column = tooltip:AddNormalHeader()
	tooltip:SetNormalCell(line, column, LEVEL..": "..UnitLevel(unit)..KEY_COMMA.." "..ITEMS.." "..LEVEL..": "..equip_level, "CENTER", 3)
	tooltip:AddSeparator()
	line, column = tooltip:AddNormalHeader("", RATING, ADDON_MISSING)

	tooltip:SetNormalCell(line, 1, ITEMS, "CENTER")
	tooltip:AddSeparator()

	local two_handed = nil
	local prev_stype = nil

	for idx, slot in ipairs(ORDERED_EQUIPMENT) do
		local item = char_equip[slot]

		rating_tot = rating_tot + item.rating

		if item.link then
			local r, g, b, hex = GetItemQualityColor(item.rarity)

			line, column = tooltip:AddNormalLine()

			if prefs.coloring == "Line" or prefs.coloring == "Both" then
				tooltip:SetLineColor(line, r, g, b)
			end
			local item_name = item.name

			if prefs.coloring == "Text" or prefs.coloring == "Both" then
				item_name = hex..item_name.."|r"
			end

			if not item.info or not item.info.bonuses then
				tooltip:SetNormalCell(line, 1, "|T"..item.texture..":22:22|t"..item_name)
			else
				tooltip:SetCell(line, 1, "|T"..item.texture..":22:22|t"..item_name, item.info.bonuses)
			end
			tooltip:SetNormalCell(line, 2, format("%.2f", item.rating))
			tooltip:SetNormalCell(line, 3, GetItemStatus(item))
			prev_stype = item.stype
		else
			local two_handed = slot == "SecondaryHand" and TWO_HANDED_WEAPON[prev_stype]

			if not two_handed  then
				line, column = tooltip:AddLine()

				if not item.empty_ok then
					tooltip:SetLineColor(line, 1)
				end
				tooltip:SetNormalCell(line, 1, EQUIPMENT[slot].name, nil, nil, 1, nil, 17)
				tooltip:SetNormalCell(line, 2, format("%.2f", item.rating))
				tooltip:SetNormalCell(line, 3, GetItemStatus(item))
				prev_stype = nil
			end
		end
	end
	tooltip:AddSeparator()

	line, column = tooltip:AddLine()

	tooltip:SetNormalCell(line, 2, format("%.2f", rating_tot))
	tooltip:Show()
end

local function ShowTooltip(self)
	BuildTooltip(self)
end

-------------------------------------------------------------------------------
-- Initialization functions
-------------------------------------------------------------------------------
function GearGauge:OnInitialize()
	-- Set the numeric ID and texture strings for the equipment slots
	for slot in pairs(EQUIPMENT) do
		EQUIPMENT[slot].id, EQUIPMENT[slot].texture = GetInventorySlotInfo(slot.."Slot")
	end

	-- Determine if the toon is an enchanter
	local enchanting_sklname = GetSpellInfo(7411)
	for idx = 1, GetNumSkillLines() do
		local skill = GetSkillLineInfo(idx)
		if skill == enchanting_sklname then
			IS_ENCHANTER = true
			break
		end
	end

	DataObj = LDB:NewDataObject("GearGauge", {
		type	= "data source",
		label	= "GearGauge",
		text	= "0",
		icon	= "Interface\\Icons\\INV_Shield_36",
		OnEnter = function(self) ShowTooltip(self) end,
		OnLeave = function() end,
		OnClick = function(button)
				  local options_frame = InterfaceOptionsFrame
				  if options_frame:IsVisible() then
					  options_frame:Hide()
				  else
					  InterfaceOptionsFrame_OpenToCategory(GearGauge.optionsFrame)
				  end
			  end,
	})

	local defaults = {
		profile = {
			scanning = {
				delay		= 10,
				persistence		= 30,
				maxlevel	= MAX_PLAYER_LEVEL,
				minlevel	= 1,
				notify		= true,
				enemies	= true,
			},
			tooltip = {
				coloring	= "Both",
				timer		= 0.25,
			},
		},
		realm = {
			entries = {},		-- Scanned players.
		},
	}
	db = LibStub("AceDB-3.0"):New(NAME.."DB", defaults)

	self:SetupOptions()
	self:RegisterChatCommand("geargauge", "SlashCommand")
	self.OnInitialize = nil
end

function GearGauge:OnEnable()
	UpdateData()
	GearGauge:RegisterEvent("PLAYER_LEVEL_UP")
	GearGauge:RegisterEvent("PLAYER_TARGET_CHANGED")
	GearGauge:RegisterEvent("UNIT_INVENTORY_CHANGED")
end

function GearGauge:OnDisable()
	GearGauge:UnregisterEvent("PLAYER_LEVEL_UP")
	GearGauge:UnregisterEvent("PLAYER_TARGET_CHANGED")
	GearGauge:UnregisterEvent("UNIT_INVENTORY_CHANGED")
end

-------------------------------------------------------------------------------
-- Configuration
-------------------------------------------------------------------------------
local GetOptions, GetScanOptions
do
	local TOOLTIP_COLORING_VALUES = {
		["Line"]	= L["Line"],
		["Text"]	= L["Text"],
		["Both"]	= L["Line and Text"],
	}
	local options, scan_options

	function GetOptions()
		if options then return options end

		options = {
			type	= "group",
			name	= NAME,
			args = {
				about = {
					order	= 1,
					type	= "description",
					name	= GAME_VERSION_LABEL.." "..version,
				},
				-------------------------------------------------------------------------------
				-- Scanning options.
				-------------------------------------------------------------------------------
				scanning = {
					order	= 2,
					type	= "group",
					name	= L["Scanning"],
					desc	= L["SCANNING_OPTIONS_DESC"],
					args = {
						scanning_desc = {
							order	= 1,
							type	= "description",
							name	= L["SCANNING_OPTIONS_DESC"].."\n",
						},
						minlev = {
							order	= 2,
							type	= "range",
							name	= L["Minimum Level"],
							desc	= L["SCAN_MINLEVEL_DESC"],
							min	= 1,
							max	= MAX_PLAYER_LEVEL,
							step	= 1,
							get	= function() return db.profile.scanning.minlevel end,
							set	= function(info, val)
									  db.profile.scanning.minlevel = math.max(1, math.min(db.profile.scanning.maxlevel, val))
								  end,
						},
						maxlev = {
							order	= 3,
							type	= "range",
							name	= L["Maximum Level"],
							desc	= L["SCAN_MAXLEVEL_DESC"],
							min	= 1,
							max	= MAX_PLAYER_LEVEL,
							step	= 1,
							get	= function() return db.profile.scanning.maxlevel end,
							set	= function(info, val)
									  db.profile.scanning.maxlevel = math.min(MAX_PLAYER_LEVEL, math.max(db.profile.scanning.minlevel, val))
								  end,
						},
						delay = {
							order	= 4,
							type	= "range",
							name	= L["Delay"],
							desc	= L["SCAN_DELAY_DESC"],
							min	= 1,
							max	= 60,
							step	= 1,
							get	= function() return db.profile.scanning.delay end,
							set	= function(info, val) db.profile.scanning.delay = math.min(60, math.max(1, val)) end,
						},
						persistence = {
							order	= 5,
							type	= "range",
							name	= L["Persistence"],
							desc	= L["SCAN_PERSISTENCE_DESC"],
							min	= 1,
							max	= 30,
							step	= 1,
							get	= function() return db.profile.scanning.persistence end,
							set	= function(info, val) db.profile.scanning.persistence = math.min(30, math.max(1, val)) end,
						},
						spacer_1 = {
							order		= 6,
							type		= "description",
							name		= " ",
							cmdHidden	= true
						},
						notify = {
							order	= 7,
							type	= "toggle",
							name	= L["Notify"],
							desc	= L["SCAN_NOTIFY_DESC"],
							get	= function() return db.profile.scanning.notify end,
							set	= function() db.profile.scanning.notify = not db.profile.scanning.notify end,
						},
						enemies = {
							order	= 8,
							type	= "toggle",
							name	= L["Scan Enemies"],
							desc	= L["SCAN_ENEMIES_DESC"],
							get	= function() return db.profile.scanning.enemies end,
							set	= function() db.profile.scanning.enemies = not db.profile.scanning.enemies end,
						},
					},
				},
				-------------------------------------------------------------------------------
				-- Tooltip options.
				-------------------------------------------------------------------------------
				tooltip = {
					order	= 3,
					type	= "group",
					name	= L["Tooltip"],
					desc	= L["TOOLTIP_OPTIONS_DESC"],
					args = {
						tooltip_desc = {
							order	= 1,
							type	= "description",
							name	= L["TOOLTIP_OPTIONS_DESC"].."\n",
						},
--						scale = {
--							order	= 1,
--							type	= "range",
--							name	= L["Scale"],
--							desc	= L["TOOLTIP_SCALE_DESC"],
--							min	= 0.5,
--							max	= 2.0,
--							step	= 0.05,
--							get	= function() return db.profile.tooltip.scale end,
--							set	= function(info, val) db.profile.tooltip.scale = math.min(2.0, math.max(0.5, val)) end,
--						},
						timer = {
							order	= 2,
							type	= "range",
							name	= L["Timer"],
							desc	= L["TOOLTIP_TIMER_DESC"],
							min	= 0.25,
							max	= 3.0,
							step	= 0.05,
							get	= function() return db.profile.tooltip.timer end,
							set	= function(info, val) db.profile.tooltip.timer = math.min(3.0, math.max(0.25, val)) end,
						},
						coloring = {
							order	= 3,
							type	= "select",
							name	= L["Coloring"],
							desc	= L["TOOLTIP_COLORING_DESC"],
							get	= function() return db.profile.tooltip.coloring end,
							set	= function(info, val) db.profile.tooltip.coloring = val end,
							values	= TOOLTIP_COLORING_VALUES,
						},
					},
				},
			},
		}
		return options
	end

	function GearGauge:SetupOptions()
		LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable(NAME, GetOptions())
		self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(NAME)
	end
end	-- do

function GearGauge:SlashCommand(arg)
	if not arg or (arg and arg:trim() == "") then
		InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
	else
		self:Print("Nothing to do yet.")
	end
end
